import { RuleConfig, MergeIntersection, RenamePrefix, VitestRules, YmlRules, NRules, Prefix, ImportRules, EslintRules, JsoncRules, VueRules, EslintCommentsRules, FlatESLintConfigItem } from '@antfu/eslint-define-config';
import { RuleOptions } from '@eslint-types/typescript-eslint/types';
import { RuleOptions as RuleOptions$1 } from '@eslint-types/unicorn/types';
import { ParserOptions } from '@typescript-eslint/parser';
import { Linter } from 'eslint';
import { FlatGitignoreOptions } from 'eslint-config-flat-gitignore';
import { Rules as Rules$1 } from 'eslint-plugin-antfu';
import { Options } from 'eslint-processor-vue-blocks';

type WrapRuleConfig<T extends {
    [key: string]: any;
}> = {
    [K in keyof T]: T[K] extends RuleConfig ? T[K] : RuleConfig<T[K]>;
};
type Awaitable<T> = T | Promise<T>;
type Rules = WrapRuleConfig<MergeIntersection<RenamePrefix<RuleOptions, '@typescript-eslint/', 'ts/'> & RenamePrefix<VitestRules, 'vitest/', 'test/'> & RenamePrefix<YmlRules, 'yml/', 'yaml/'> & RenamePrefix<NRules, 'n/', 'node/'> & Prefix<Rules$1, 'antfu/'> & ImportRules & EslintRules & JsoncRules & VueRules & RuleOptions$1 & EslintCommentsRules & {
    'test/no-only-tests': RuleConfig<[]>;
}>>;
type FlatConfigItem = Omit<FlatESLintConfigItem<Rules, false>, 'plugins'> & {
    /**
     * Custom name of each config item
     */
    name?: string;
    /**
     * An object containing a name-value mapping of plugin names to plugin objects. When `files` is specified, these plugins are only available to the matching files.
     *
     * @see [Using plugins in your configuration](https://eslint.org/docs/latest/user-guide/configuring/configuration-files-new#using-plugins-in-your-configuration)
     */
    plugins?: Record<string, any>;
};
type UserConfigItem = FlatConfigItem | Linter.FlatConfig;
type OptionsFiles = {
    /**
     * Override the `files` option to provide custom globs.
     */
    files?: string[];
};
type OptionsVue = {
    /**
     * Create virtual files for Vue SFC blocks to enable linting.
     *
     * @see https://github.com/antfu/eslint-processor-vue-blocks
     * @default true
     */
    sfcBlocks?: boolean | Options;
} & OptionsOverrides;
type OptionsTypescript = (OptionsTypeScriptWithTypes & OptionsOverrides) | (OptionsTypeScriptParserOptions & OptionsOverrides);
type OptionsComponentExts = {
    /**
     * Additional extensions for components.
     *
     * @example ['vue']
     * @default []
     */
    componentExts?: string[];
};
type OptionsTypeScriptParserOptions = {
    /**
     * Additional parser options for TypeScript.
     */
    parserOptions?: Partial<ParserOptions>;
    /**
     * Glob patterns for files that should be type aware.
     * @default ['**\/*.{ts,tsx}']
     */
    filesTypeAware?: string[];
};
type OptionsTypeScriptWithTypes = {
    /**
     * When this options is provided, type aware rules will be enabled.
     * @see https://typescript-eslint.io/linting/typed-linting/
     */
    tsconfigPath?: string | string[];
};
type OptionsHasTypeScript = {
    typescript?: boolean;
};
type OptionsOverrides = {
    overrides?: FlatConfigItem['rules'];
};
type OptionsIsInEditor = {
    isInEditor?: boolean;
};
type OptionsConfig = {
    /**
     * Enable gitignore support.
     *
     * Passing an object to configure the options.
     *
     * @see https://github.com/antfu/eslint-config-flat-gitignore
     * @default true
     */
    gitignore?: boolean | FlatGitignoreOptions;
    /**
     * Core rules. Can't be disabled.
     */
    javascript?: OptionsOverrides;
    /**
     * Enable TypeScript support.
     *
     * Passing an object to enable TypeScript Language Server support.
     *
     * @default auto-detect based on the dependencies
     */
    typescript?: boolean | OptionsTypescript;
    /**
     * Enable test support.
     *
     * @default true
     */
    test?: boolean | OptionsOverrides;
    /**
     * Enable Vue support.
     *
     * @default auto-detect based on the dependencies
     */
    vue?: boolean | OptionsVue;
    /**
     * Enable JSONC support.
     *
     * @default true
     */
    jsonc?: boolean | OptionsOverrides;
    /**
     * Enable YAML support.
     *
     * @default true
     */
    yaml?: boolean | OptionsOverrides;
    /**
     * Control to disable some rules in editors.
     * @default auto-detect based on the process.env
     */
    isInEditor?: boolean;
    /**
     * Provide overrides for rules for each integration.
     *
     * @deprecated use `overrides` option in each integration key instead
     */
    overrides?: {
        javascript?: FlatConfigItem['rules'];
        typescript?: FlatConfigItem['rules'];
        test?: FlatConfigItem['rules'];
        vue?: FlatConfigItem['rules'];
        yaml?: FlatConfigItem['rules'];
    };
} & OptionsComponentExts;

type ResolvedOptions<T> = T extends boolean ? never : NonNullable<T>;
declare const resolveSubOptions: <K extends keyof OptionsConfig>(options: OptionsConfig, key: K) => ResolvedOptions<OptionsConfig[K]>;
declare const getOverrides: <K extends keyof OptionsConfig>(options: OptionsConfig, key: K) => any;
/**
 * Construct an array of ESLint flat config items.
 *
 * @param {OptionsConfig & FlatConfigItem} options
 *  The options for generating the ESLint configurations.
 * @param {Awaitable<UserConfigItem | UserConfigItem[]>[]} userConfigs
 *  The user configurations to be merged with the generated configurations.
 * @returns {Promise<UserConfigItem[]>}
 *  The merged ESLint configurations.
 */
declare const sjkeyConfig: (options?: OptionsConfig & FlatConfigItem, ...userConfigs: Array<Awaitable<UserConfigItem | UserConfigItem[]>>) => Promise<UserConfigItem[]>;

declare const comments: () => Promise<FlatConfigItem[]>;

declare const ignores: () => Promise<FlatConfigItem[]>;

declare const imports: () => Promise<FlatConfigItem[]>;

declare const javascript: (options?: OptionsIsInEditor & OptionsOverrides) => Promise<FlatConfigItem[]>;

declare const node: () => Promise<FlatConfigItem[]>;

declare const jsonc: (options?: OptionsFiles & OptionsOverrides) => Promise<FlatConfigItem[]>;

/**
 * Sort package.json
 *
 * Requires `jsonc` config
 */
declare const sortPackageJson: () => Promise<FlatConfigItem[]>;
/**
 * Sort tsconfig.json
 *
 * Requires `jsonc` config
 */
declare const sortTsconfig: () => FlatConfigItem[];

declare const test: (options?: OptionsFiles & OptionsIsInEditor & OptionsOverrides) => Promise<FlatConfigItem[]>;

declare const typescript: (options?: OptionsFiles & OptionsComponentExts & OptionsOverrides & OptionsTypeScriptWithTypes & OptionsTypeScriptParserOptions) => Promise<FlatConfigItem[]>;

declare const unicorn: () => Promise<FlatConfigItem[]>;

declare const vue: (options?: OptionsVue & OptionsOverrides & OptionsFiles) => Promise<FlatConfigItem[]>;

declare const yaml: (options?: OptionsOverrides & OptionsFiles) => Promise<FlatConfigItem[]>;

declare const prettier: () => Promise<FlatConfigItem[]>;

declare const stylistic: () => Promise<FlatConfigItem[]>;

declare const GLOB_SRC_EXT = "?([cm])[jt]s?(x)";
declare const GLOB_SRC = "**/*.?([cm])[jt]s?(x)";
declare const GLOB_JS = "**/*.?([cm])js";
declare const GLOB_TS = "**/*.?([cm])ts";
declare const GLOB_JSON = "**/*.json";
declare const GLOB_JSON5 = "**/*.json5";
declare const GLOB_JSONC = "**/*.jsonc";
declare const GLOB_VUE = "**/*.vue";
declare const GLOB_YAML = "**/*.y?(a)ml";
declare const GLOB_TESTS: string[];
declare const GLOB_EXCLUDE: string[];

export { type Awaitable, type FlatConfigItem, GLOB_EXCLUDE, GLOB_JS, GLOB_JSON, GLOB_JSON5, GLOB_JSONC, GLOB_SRC, GLOB_SRC_EXT, GLOB_TESTS, GLOB_TS, GLOB_VUE, GLOB_YAML, type OptionsComponentExts, type OptionsConfig, type OptionsFiles, type OptionsHasTypeScript, type OptionsIsInEditor, type OptionsOverrides, type OptionsTypeScriptParserOptions, type OptionsTypeScriptWithTypes, type OptionsTypescript, type OptionsVue, type ResolvedOptions, type Rules, type UserConfigItem, type WrapRuleConfig, comments, sjkeyConfig as default, getOverrides, ignores, imports, javascript, jsonc, node, prettier, resolveSubOptions, sjkeyConfig, sortPackageJson, sortTsconfig, stylistic, test, typescript, unicorn, vue, yaml };
